IMAGE_START_OFFSET = 161

# Definition of all the vocabulary used
ABLATIONDISTANCEBETWEENSHOTSX = "AblationDistanceBetweenShotsX"
ABLATIONDISTANCEBETWEENSHOTSY = "AblationDistanceBetweenShotsY"
ABLATIONFREQUENCY = "AblationFrequency"
ABLATIONPOWER = "AblationPower"
ACQUISITION = "Acquisition"
ACQUISITIONCHANNEL = "AcquisitionChannel"
ACQUISITIONID = "AcquisitionID"
ACQUISITIONROI = "AcquisitionROI"
ACQUISITIONROIID = "AcquisitionROIID"
AFTERABLATIONIMAGEENDOFFSET = "AfterAblationImageEndOffset"
AFTERABLATIONIMAGESTARTOFFSET = "AfterAblationImageStartOffset"
BEFOREABLATIONIMAGEENDOFFSET = "BeforeAblationImageEndOffset"
BEFOREABLATIONIMAGESTARTOFFSET = "BeforeAblationImageStartOffset"
CHANNELLABEL = "ChannelLabel"
CHANNELNAME = "ChannelName"
DATAENDOFFSET = "DataEndOffset"
DATASTARTOFFSET = "DataStartOffset"
DESCRIPTION = "Description"
DUALCOUNTSTART = "DualCountStart"
ENDTIMESTAMP = "EndTimeStamp"
FILENAME = "Filename"
HEIGHTUM = "HeightUm"
ID = "ID"
IMAGEENDOFFSET = "ImageEndOffset"
IMAGEFILE = "ImageFile"
IMAGEFORMAT = "ImageFormat"
IMAGESTARTOFFSET = "ImageStartOffset"
MCDSCHEMA = "MCDSchema"
MAXX = "MaxX"
MAXY = "MaxY"
MOVEMENTTYPE = "MovementType"
ORDERNUMBER = "OrderNumber"
PANORAMA = "Panorama"
PANORAMAID = "PanoramaID"
PANORAMAPIXELXPOS = "PanoramaPixelXPos"
PANORAMAPIXELYPOS = "PanoramaPixelYPos"
PIXELHEIGHT = "PixelHeight"
PIXELSCALECOEF = "PixelScaleCoef"
PIXELWIDTH = "PixelWidth"
PLUMEEND = "PlumeEnd"
PLUMESTART = "PlumeStart"
ROIENDXPOSUM = "ROIEndXPosUm"
ROIENDYPOSUM = "ROIEndYPosUm"
ROIPOINT = "ROIPoint"
ROISTARTXPOSUM = "ROIStartXPosUm"
ROISTARTYPOSUM = "ROIStartYPosUm"
ROITYPE = "ROIType"
SEGMENTDATAFORMAT = "SegmentDataFormat"
SIGNALTYPE = "SignalType"
SLIDE = "Slide"
SLIDEID = "SlideID"
SLIDETYPE = "SlideType"
SLIDEX1POSUM = "SlideX1PosUm"
SLIDEX2POSUM = "SlideX2PosUm"
SLIDEX3POSUM = "SlideX3PosUm"
SLIDEX4POSUM = "SlideX4PosUm"
SLIDEXPOSUM = "SlideXPosUm"
SLIDEY1POSUM = "SlideY1PosUm"
SLIDEY2POSUM = "SlideY2PosUm"
SLIDEY3POSUM = "SlideY3PosUm"
SLIDEY4POSUM = "SlideY4PosUm"
SLIDEYPOSUM = "SlideYPosUm"
STARTTIMESTAMP = "StartTimeStamp"
TEMPLATE = "Template"
UID = "UID"
VALUEBYTES = "ValueBytes"
WIDTHUM = "WidthUm"
SWVERSION = "SwVersion"
